#ifndef HELLOSPLINE_H
#define HELLOSPLINE_H

#include "scene.h"
#include "hello_camera/camera.h"
#include "mainwindow.h"

#include <memory>
#include <functional>

/** Simple drawing demonstration
 */
class SimpleSpline : public Scene {
public:
    explicit SimpleSpline(int width, int height, MainWindow* w);
    ~SimpleSpline();

    void resize(int width, int height) override;
    void draw() override;

    void mouseclick(int button, float xpos, float ypos) override;
    void mousemove(float xpos, float ypos) override;
    void keyboardmove(int key, double time) override;
    bool keyboard(unsigned char k) override;

public slots:
    void generateProgram();

private:
    // A simple geometry
    std::vector<GLfloat> _vertices;
    std::vector<GLfloat> _normals;
    std::vector<GLuint> _indices;

    // OpenGL object for geometry
    GLuint _vao;
    GLuint _vbo;
    GLuint _nbo;
    GLuint _ebo;

    // Shader program for rendering
    QString _lastVertex;
    QString _lastFragment;
    GLuint _program;

    // for mouse management
    int _button; // 0 --> left. 1 --> right. 2 --> middle. 3 --> other
    float _mousex{0};
    float _mousey{0};

    // Camera
    using CameraSelector=std::function<Camera*()>;
    std::vector<CameraSelector> _cameraselector;
    int _activecamera;
    std::unique_ptr<Camera> _camera;

    // Matrices
    glm::mat4 _model;
    glm::mat4 _view;
    glm::mat4 _projection;

    /* Shape Methods
    void drawCube(float size = 0.1f);
    void drawUVSphere(int resolution = 5, float radius = 0.5f);
    void drawIcoSphere(int resolution = 5, float radius = 0.f);
    float _radius = 0.5f;
    float _errorMax = 0.0f;
    */
};

#endif // SIMPLESPLINE_H
