#version 410 core
in      vec3 position;
in      vec3 normal;

uniform vec3 cameraView;

out     vec4 color;

// Couleurs
vec3 materialColor  = vec3( 0.4,   1,  0.6);
vec3 specularColor  = vec3(   1,    1,   1);
vec3 lightColor     = vec3(0.95, 0.98,   1);

// Position de la lumière
vec3 frontLightPos  = vec3( 0,  0, 1);
float specCoef      = 128;


/* Bibliographie:
 * https://en.wikipedia.org/wiki/Blinn%E2%80%93Phong_shading_model
 * http://www.opengl-tutorial.org/fr/beginners-tutorials/tutorial-8-basic-shading/
 */

void main(void)
{
    vec3 frontLight = frontLightPos - position;
    float distance = pow(length(frontLight), 2);
    frontLight = normalize(frontLight);
    vec3 normalizedNormal = normalize(normal);

    // Paramètres de réflexion initiaux
    float lambertian = max(dot(frontLight, normalizedNormal), 0.0);
    float specularFactor = 0;

    /* Ambient: simule l'éclairage indirect */
    vec3 ambient = vec3(0.15, 0.15, 0.15) * materialColor;

    /* Diffuse: "couleur" de l'objet dans la zone illuminée */
    // Cosinus entre la normale et la direction de la lumière (positif)
    //  - lumière à la verticale = 1
    //  - lumière perpendiculaire ou derrière = 0
    float cosTheta = clamp( dot( normalizedNormal, frontLight ), 0, 1 );
    vec3 diffuse = materialColor * lightColor * cosTheta;

    /* Specular: réflexion */
    if (lambertian > 0.0) {
        vec3 viewDir = normalize(-cameraView);
        vec3 halfDir = normalize(frontLight + viewDir);
        float specAngle = max(dot(halfDir, normalizedNormal), 0.0);
        specularFactor = pow(specAngle, specCoef);
    }
    vec3 specular = specularColor * lightColor * specularFactor;

    /* Couleur = Ambient + Diffuse + Specular */
    color = vec4(ambient + diffuse + specular, 1);
}
